@@ -5,7 +5,7 @@ from typing import Optional, Callable, Union
 from pathlib import Path
 
 from PyQt6 import QtCore, QtGui, QtWidgets
-from PyQt6.QtCore import Qt, QtMsgType, qInstallMessageHandler
+from PyQt6.QtCore import Qt, QtMsgType, qInstallMessageHandler,QObject,pyqtSignal
 
 # ── paths ────────────────────────────────────────────────────────────────
 CACHE_DIR = os.path.join(os.path.expanduser("~"), ".cache", "abstract_finder")
@@ -19,7 +19,73 @@ _SERVICE_SINGLETON: "LogService|None" = None
 
 def get_log_file_path() -> str:
     return LOG_FILE
+class QtLogEmitter(QObject):
+    new_log = pyqtSignal(str)
 
+_emitter_singleton: QtLogEmitter | None = None
+_handler_singleton: logging.Handler | None = None
+
+def _emitter() -> QtLogEmitter:
+    global _emitter_singleton
+    if _emitter_singleton is None:
+        _emitter_singleton = QtLogEmitter()
+    return _emitter_singleton
+
+class QtLogHandler(logging.Handler):
+    def __init__(self, emitter: QtLogEmitter):
+        super().__init__()
+        self.emitter = emitter
+    def emit(self, record: logging.LogRecord):
+        try:
+            msg = self.format(record)
+        except Exception:
+            msg = record.getMessage()
+        self.emitter.new_log.emit(msg + "\n")
+
+def install_python_logging():
+    root = logging.getLogger()
+    if not root.handlers:
+        root.setLevel(logging.DEBUG)
+
+        f = RotatingFileHandler(LOG_FILE, maxBytes=5_000_000, backupCount=5, encoding="utf-8")
+        f.setLevel(logging.DEBUG)
+        f.setFormatter(logging.Formatter("%(asctime)s [%(levelname)s] %(name)s:%(lineno)d - %(message)s"))
+        root.addHandler(f)
+
+        c = logging.StreamHandler(sys.stderr)
+        c.setLevel(logging.INFO)
+        c.setFormatter(logging.Formatter("%(asctime)s [%(levelname)s] %(message)s"))
+        root.addHandler(c)
+
+    def _format_exc(exctype, value, tb):
+        return "".join(traceback.format_exception(exctype, value, tb))
+    def excepthook(exctype, value, tb):
+        logging.critical("UNCAUGHT EXCEPTION:\n%s", _format_exc(exctype, value, tb))
+    sys.excepthook = excepthook
+
+def install_qt_bridge():
+    global _handler_singleton
+    if _handler_singleton is None:
+        _handler_singleton = QtLogHandler(_emitter())
+        _handler_singleton.setLevel(logging.DEBUG)
+        _handler_singleton.setFormatter(logging.Formatter("%(asctime)s [%(levelname)s] %(message)s"))
+        logging.getLogger().addHandler(_handler_singleton)
+
+    def qt_msg(mode, ctx, message):
+        level = {
+            QtMsgType.QtDebugMsg:    logging.DEBUG,
+            QtMsgType.QtInfoMsg:     logging.INFO,
+            QtMsgType.QtWarningMsg:  logging.WARNING,
+            QtMsgType.QtCriticalMsg: logging.ERROR,
+            QtMsgType.QtFatalMsg:    logging.CRITICAL,
+        }.get(mode, logging.INFO)
+        logging.log(level, "Qt: %s (%s:%d)", message, ctx.file, ctx.line)
+    qInstallMessageHandler(qt_msg)
+
+def install_qt_logging():
+    """Call once at app startup (idempotent)."""
+    install_python_logging()
+    install_qt_bridge()
 # ── core logging wiring ─────────────────────────────────────────────────
 def setup_root_logging(level=logging.DEBUG, stderr_level=logging.INFO) -> None:
     global _LOGGING_WIRED
@@ -168,9 +234,10 @@ def get_log_service() -> LogService:
         _SERVICE_SINGLETON = LogService()
     return _SERVICE_SINGLETON
 
-# optional mini runner
-def startConsole(widget_or_cls: Union[type[QtWidgets.QWidget], QtWidgets.QWidget], *args, **kwargs) -> int:
-    app = QtWidgets.QApplication.instance() or QtWidgets.QApplication(sys.argv)
-    win = widget_or_cls(*args, **kwargs) if isinstance(widget_or_cls, type) else widget_or_cls
-    win.show()
-    return app.exec()
+### optional mini runner
+##def startConsole(widget_or_cls: Union[type[QtWidgets.QWidget], QtWidgets.QWidget], *args, **kwargs) -> int:
+##    app = QtWidgets.QApplication.instance() or QtWidgets.QApplication(sys.argv)
+##    win = widget_or_cls(*args, **kwargs) if isinstance(widget_or_cls, type) else widget_or_cls
+##    win.show()
+##    return app.exec()
+##
